/**
 * JASMINe
 * Copyright (C) 2010-2011 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.view {
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.display.DisplayObjectContainer;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.events.TimerEvent;
import flash.geom.Point;
import flash.net.FileFilter;
import flash.utils.Dictionary;
import flash.utils.Timer;

import mx.charts.series.AreaSeries;
import mx.charts.series.LineSeries;
import mx.collections.ArrayCollection;
import mx.containers.Canvas;
import mx.containers.HBox;
import mx.containers.TitleWindow;
import mx.containers.VBox;
import mx.controls.*;
import mx.events.*;
import mx.graphics.Stroke;
import mx.managers.PopUpManager;
import mx.resources.ResourceManager;
import mx.rpc.events.FaultEvent;

import org.ow2.jasmine.monitoring.eos.common.controls.DateTimePicker;
import org.ow2.jasmine.monitoring.eos.common.events.LayoutChangeEvent;
import org.ow2.jasmine.monitoring.eos.common.events.LoadEvent;
import org.ow2.jasmine.monitoring.eos.common.events.PodMenuEvent;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.common.managers.PodLayoutManager;
import org.ow2.jasmine.monitoring.eos.common.view.AreaChartView;
import org.ow2.jasmine.monitoring.eos.common.view.CartesianChartClass;
import org.ow2.jasmine.monitoring.eos.common.view.LineChartView;
import org.ow2.jasmine.monitoring.eos.common.view.LoadWindow;
import org.ow2.jasmine.monitoring.eos.common.view.Pod;
import org.ow2.jasmine.monitoring.eos.monitoring.event.*;
import org.ow2.jasmine.monitoring.eos.monitoring.model.*;
import org.ow2.jasmine.monitoring.eos.monitoring.vo.*;

/**
 * This is the action script part of the Monitoring EoS module
 */
[Bindable]
public class DashboardClass extends VBox {

    protected var model:MonitoringModelLocator = MonitoringModelLocator.getInstance();
    protected var dispatcher:CairngormEventDispatcher = CairngormEventDispatcher.getInstance();

    // dashboard Id.
    protected var did:int = -1;

    protected var podLayoutManager:PodLayoutManager = null;

    // series informations: [serId] -> Serie
    protected var seriesInfo:Dictionary = null;

    // series data providers: [serId] -> ArrayCollection[date:Date, value:String]
    protected var seriesDataProviders:Dictionary = null;

    /**
     * Current Pod Id in case of menu action
     * this Id is used to index all the following Dictionnaries
     */
    protected var podId:String;

    /**
     * List of ChartBase used in this dashboard
     * graphCharts[podId] -> CartesianChartClass
     */
    protected var graphCharts:Dictionary = null;

    /**
     * List of Pods
     *  graphPods[podId] -> Pod
     */
    protected var graphPods:Dictionary = null;

    // for each graph, give the list of series
    // [graph.id] -> Array[serie.id]
    // needed when changing types of graphs
    protected var dataProviderTable:Dictionary = null;

    // type of the graph (area, line)
    protected static const LINE_CHART:int = 0;
    protected static const AREA_CHART:int = 1;

    protected var databaseFrom:Date;
    protected var databaseTo:Date;

    protected var INDEX_GRAPH_IN_POD = 0;

    // Config File, with params replaced by their value
    public var strConfigFile:String = "";

    public var replayFileName:String;

    // true if mode replay, false if live monitoring.
    protected var replayMode:Boolean = false;

    // In replay mode: true if replay from database.
    protected var databaseReplay:Boolean = false;

    // Last state before reading new data
    protected var lastState:String;

    /**
     * For each podId, zoomOn is true if zoom has been selected.
     */
    protected var zoomOn:Dictionary = new Dictionary();

    // -------------------------------------------------------------------
    // These variables are defined in the mxml part.
    // -------------------------------------------------------------------

    [Bindable]
    public var headerLine2:HBox;

    [Bindable]
    public var prevButton:LinkButton;

    [Bindable]
    public var nextButton:LinkButton;

    [Bindable]
    public var pauseButton:LinkButton;

    [Bindable]
    public var startButton:LinkButton;

    [Bindable]
    public var stopButton:LinkButton;

    [Bindable]
    public var graphsCanvas:Canvas;

    [Bindable]
    public var dateFrom:DateTimePicker;

    [Bindable]
    public var dateTo:DateTimePicker;

    [Bindable]
    public var applyZoomButton:LinkButton;

    [Bindable]
    public var resetZoomButton:LinkButton;

    [Bindable]
    public var refreshTimeSlider:HSlider;

    [Bindable]
    public var modeTextInput:TextInput;

    // -------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------

    public function DashboardClass() {
    }

    public function init(strConfigFile:String) {
        this.strConfigFile = strConfigFile;
    }

    public function initComponent():void {
        // Listen to server side events
        dispatcher.addEventListener(ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION, ServerSideExceptionEvent.show);

        // parse XML configuration
        if (! parseXmlFile()) {
            model.util.removeTab(this as DashboardView);
            return;
        }

        // Unset the did, because it has been initialized with a bad value from model
        // (actually: the last current value)
        // This is done when this object is built, because of the binding.
        this.did = -3;

        // Ask the Server to create a new dashboard with this list of series.
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.CREATE_DASHBOARD + model.componentID);
        ev.series = new ArrayCollection();
        for each (var s:Serie in this.seriesInfo) {
            ev.series.addItem(s);
        }
        dispatcher.dispatchEvent(ev);
    }

    /**
     * This is needed only for localization of the label.
     * @return
     */
    protected function getSliderLabels():Array {
        var str_array:Array = new Array();
        str_array[0] = "Os";
        str_array[1] = locale('refresh_period');
        str_array[2] = "60s";
        return  str_array;
    }

    // -------------------------------------------------------------------
    // Main Menu (Actions)
    // -------------------------------------------------------------------

    protected var mainMenu:Menu = null;

    /**
     * Open the menu
     */
    protected function mainMenuOpen(e:MouseEvent):void {
        mainMenu = Menu.createMenu(null, mainMenuData, false);
        mainMenu.addEventListener(MenuEvent.ITEM_CLICK, mainMenuClick);

        // get the menu parent from the mouse event
        var target:DisplayObjectContainer = DisplayObjectContainer(e.currentTarget);

        // position menu relative to the parent
        var point:Point = new Point(e.localX, e.localY);
        point = target.localToGlobal(point);

        // show the menu just below the link button
        mainMenu.show(point.x - 5, point.y + 16);
    }

    /**
     * Close the Menu
     */
    protected function mainMenuClose(e:MouseEvent):void {
        mainMenu.hide();
        mainMenu = null;
    }

    protected var mainMenuData:Array = [
        {label: "mode", children: [
            {label: locale("mainMenuData_live"), type: "radio", id: "live", toggled: true},
            {label: locale("mainMenuData_rserv"), type: "radio", id: "rserv"},
            {label: locale("mainMenuData_rloc"), type: "radio", id: "rloc"},
            {label: locale("mainMenuData_rdb"), type: "radio", id: "rdb"}
        ]},
        {label: "options", children: [
            {label: locale("mainMenuData_zoom"), type: "check", id: "zoom", toggled: false},
            {label: locale("mainMenuData_follow"), type: "check", toggled: false, id: "follow"},
        ]},
        {label: locale("mainMenuData_rename"), type: "normal", id: "rename"},
        {label: locale("mainMenuData_quit"), type: "normal", id: "quit"}
    ];

    public function toggleMainMenu(e:MouseEvent):void {
        if (mainMenu != null) {
            mainMenuClose(e);
        }
        else {
            mainMenuOpen(e);
        }
    }

    /**
     * Main Menu has been used
     * @param menuEvent
     */
    protected function mainMenuClick(menuEvent:MenuEvent):void {
        mainMenu = null;
        var id:String = menuEvent.item.id;
        if (id == "follow") {
            for each (var ccc:CartesianChartClass in this.graphCharts) {
                ccc.followMonitoring(menuEvent.item.toggled);
            }
        }
        else if (id == "zoom") {
            if (menuEvent.item.toggled) {
                this.headerLine2.height = 30;
            }
            else {
                this.headerLine2.height = 0;
            }
        }
        else if (id == "live") {
            // reset all data
            flushProviders();
            // switch mode to "live monitoring"
            liveMonitoring();
        }
        else if (id == "rserv") {
            // switch mode to "replay from server file"
            stopLiveMonitoring();
            // reset all data
            flushProviders();
            this.databaseReplay = false;
            var rwin:ReplayWindow = PopUpManager.createPopUp(this.graphsCanvas, ReplayWindow, true) as ReplayWindow;
            rwin.addEventListener(LoadEvent.LOADED, replayFile);
            rwin.init(this);
            PopUpManager.centerPopUp(rwin);
        }
        else if (id == "rloc") {
            stopLiveMonitoring();
            // reset all data
            flushProviders();
            this.databaseReplay = false;
            // switch mode to "replay from local file"
            var lwin:LoadWindow = PopUpManager.createPopUp(this.graphsCanvas, LoadWindow, true) as LoadWindow;
            var xmlTypes:FileFilter = new FileFilter("Log files (*.log, *.txt, *csv)", "*.log;*.txt;*.csv");
            lwin.fileFilter = new Array(xmlTypes);
            lwin.title = locale("lwin_title");
            lwin.addEventListener(LoadEvent.LOADED, replayFile);
            PopUpManager.centerPopUp(lwin);
        }
        else if (id == "rdb") {
            stopLiveMonitoring();
            // reset all data
            flushProviders();
            this.databaseReplay = true;
            // switch mode to "replay from database"
            var dbwin:DatabaseWindow = PopUpManager.createPopUp(this.graphsCanvas, DatabaseWindow, true) as DatabaseWindow;
            dbwin.init(this);
            PopUpManager.centerPopUp(dbwin);
        }
        else if (id == "rename") {
            // rename dashboard
            var gnwin:DashboardOptions = PopUpManager.createPopUp(this.graphsCanvas, DashboardOptions, true) as DashboardOptions;
            gnwin.title = locale("dopt_title");
            gnwin.init(this);
            PopUpManager.centerPopUp(gnwin);
        }
        else if (id == "quit") {
            stopLiveMonitoring();
            // Notify server that this dashboard is removed
            removeMe();
        }
        else {
            Alert.show("Menu Item Clicked: " + menuEvent.item.label, "Menu Click");
        }
    }

    // -------------------------------------------------------------------
    // Pod Menu (options)
    // -------------------------------------------------------------------

    protected var podmenu:Menu = null;

    /**
     * Pod Menu has been clicked
     * @param e
     */
    protected function podMenuOpen(e:PodMenuEvent):void {
        // Keep the current podId for later use
        podId = e.podId;

        // Init the toggled elements in the menu
        // This is needed because the data menu is shared by all the pods.
        for (var i:int = 0; i < podMenuData.length; i++) {
            var elem:Object = podMenuData[i];
            if (elem.id == "zoom") {
                elem.toggled = zoomOn[podId];
            }
            else if (elem.id == "type") {
                for (var j:int = 0; j < elem.children.length; j++) {
                    var elem2 = elem.children[j];
                    if (elem2.id == "line") {
                        elem2.toggled = (graphCharts[podId] is LineChartView);
                    }
                    else if (elem2.id == "area") {
                        elem2.toggled = (graphCharts[podId] is AreaChartView);
                    }
                }
            }
        }

        podmenu = Menu.createMenu(null, podMenuData, false);
        podmenu.addEventListener(MenuEvent.ITEM_CLICK, podMenuClick);

        // get the menu parent from the mouse event
        var target:DisplayObjectContainer = DisplayObjectContainer(e.mouseEvent.currentTarget);

        // position menu relative to the parent
        var point:Point = new Point(e.mouseEvent.localX, e.mouseEvent.localY);
        point = target.localToGlobal(point);

        // show the menu just below the link button
        podmenu.show(point.x - 50, point.y + 16);
    }

    protected function podMenuClose(e:PodMenuEvent):void {
        podmenu.hide();
        podmenu = null;
    }

    protected var podMenuData:Array = [
        {label: locale("podMenu_export"), type: "normal", id: "export"},
        {label: locale("podMenu_zoom"), type: "check", id: "zoom", toggled: false},
        {label: locale("podMenu_type"), id: "type", children: [
            {label: locale("podMenu_line"), id: "line", type: "radio", toggled: false},
            {label: locale("podMenu_area"), id: "area", type: "radio", toggled: false}
        ]}
    ];

    public function togglePodMenu(e:PodMenuEvent):void {
        if (podmenu != null) {
            podMenuClose(e);
        }
        else {
            podMenuOpen(e);
        }
    }

    /**
     * Pod Menu has been used
     * @param menuEvent
     */
    protected function podMenuClick(menuEvent:MenuEvent):void {
        podmenu = null;
        var id:String = menuEvent.item.id;
        if (id == "export") {
            graphCharts[podId].export();
        }
        else if (id == "zoom") {
            graphCharts[podId].zoom(menuEvent.item.toggled);
            zoomOn[podId] = menuEvent.item.toggled;
        }
        else if (id == "line") {
            changeGraphType(LINE_CHART);
        }
        else if (id == "area") {
            changeGraphType(AREA_CHART);
        }
        else {
            Alert.show("Menu Item Clicked: " + menuEvent.item.label, "Menu Click");
        }
    }

    // -------------------------------------------------------------------
    // Setters triggered by bindings
    // -------------------------------------------------------------------

    /**
     * New dashboard created: Store its identifier, chosen by the server.
     * @param event
     */
    public function set newDashboard(id:int):void {
        if (this.did < 0 && id > 0) {
            // This dashboard has been succesfully created on server.
            this.did = id;

            // default state is "live monitoring"
            liveMonitoring();
        }
    }

    /**
     * Error from Server
     */
    public function set error(event:FaultEvent):void {
        Alert.show(locale("error_disconnected"), locale("error_occured"));
        stopTimer();
    }

    /**
     * Fill in the providers with datas.
     * @param prov Collection of EventData
     */
    public function set liveProviders(prov:ArrayCollection):void {
        // Check dashboard Id
        if (model.dbIdent != did) {
            // These data are not for me.
            if (model.dbIdent > 0) {
                // Alert.show("liveProviders Data not for me: "+ did + " != " + model.dbIdent);
            }
            return;
        }

        // If end of replay : stop the timer if it is on.
        if (this.replayMode && prov == null) {
            stopTimer();
            this.currentState = "stopped";
            return;
        }

        // send data to graphs
        liveDataLoad(prov);
        this.currentState = this.lastState;
    }

    // -------------------------------------------------------------------
    // States
    // -------------------------------------------------------------------

    /**
     * Enter state "stopped"
     */
    protected function enterStopped():void {
        startButton.enabled = true;
        startButton.label = "Restart";
        stopButton.enabled = false;
        pauseButton.enabled = false;
        nextButton.enabled = true;
        prevButton.enabled = true;
    }

    /**
     * Enter state "started":
     * The server is now collecting data.
     */
    protected function enterStarted():void {
        startButton.enabled = true;
        startButton.label = "Start";
        stopButton.enabled = true;
        pauseButton.enabled = false;
        nextButton.enabled = true;
        prevButton.enabled = true;
    }

    /**
     * Enter state "timer"
     * The timer has been started to periodically get data from the server.
     */
    protected function enterTimer():void {
        startButton.enabled = false;
        stopButton.enabled = true;
        pauseButton.enabled = true;
        nextButton.enabled = false;
        prevButton.enabled = false;
    }

    /**
     * Enter state "waiting"
     * We are waiting for new data from the server.
     */
    protected function enterWaiting():void {
        startButton.enabled = false;
        stopButton.enabled = false;
        pauseButton.enabled = false;
        nextButton.enabled = false;
        prevButton.enabled = false;
    }

    // -------------------------------------------------------------------
    // Modes of execution
    // -------------------------------------------------------------------

    /**
     * Replay from file: The File has been eviously selected by user.
     * @param event
     */
    protected function replayFile(event:LoadEvent):void {
        this.replayFileName = event.filename;

        // Display only the last part of the full path
        var maxsize:int = 32;
        var index:int = this.replayFileName.length - maxsize;
        var troncated:String = this.replayFileName;
        if (index > 0)
            troncated = this.replayFileName.substring(index);
        this.modeTextInput.text = locale("replay") + " " + troncated;
        this.replayMode = true;

        // Start reading the File on the server
        startFileReplay();
    }

    /**
     * Replay from Database.
     * This is called when getting out of the DatabaseWindow
     * @param from
     * @param to
     */
    public function replayDatabase(from:Date, to:Date):void {
        this.modeTextInput.text = locale("replay_database");
        this.replayMode = true;
        this.databaseFrom = from;
        this.databaseTo = to;

        // Start reading the database
        startDatabaseReplay();
    }

    /**
     * Set mode to live monitoring
     */
    protected function liveMonitoring():void {
        this.modeTextInput.text = locale("live_monitoring");
        this.replayMode = false;
        startLiveMonitoring();
    }

    /**
     * Pod has been created so update the respective PodLayoutManager.
     * FIXME: Should wait that ALL pods have been completed their creation.
     */
    protected function displayPods(e:FlexEvent):void {
        e.currentTarget.removeEventListener(FlexEvent.UPDATE_COMPLETE, displayPods);

        // this is really needed ! (why ?)
        this.podLayoutManager.removeNullItems();

        this.podLayoutManager.updateLayout(false);
        // ??? this.podLayoutManager.dispatchEvent(new LayoutChangeEvent(LayoutChangeEvent.UPDATE));
    }

    // -------------------------------------------------------------------
    // Buttons
    // -------------------------------------------------------------------

    /**
     * PREV
     */
    public function prevButtonClick():void {
        // TODO
    }

    /**
     * PAUSE
     */
    public function pauseButtonClick():void {
        stopTimer();
        this.currentState = "started";
    }

    /**
     * NEXT
     */
    public function nextButtonClick():void {
        if (this.currentState == "started") {
            nextPage();
        }
        else {
            // TODO
        }
    }

    /**
     * START or RESTART
     */
    public function startButtonClick():void {
        if (this.currentState == "stopped") {
            // reset all data
            flushProviders();

            // start monitoring on server
            if (replayMode) {
                if (databaseReplay) {
                   startDatabaseReplay();
                }
                else {
                   startFileReplay();
                }
            }
            else {
                startLiveMonitoring();
            }
        }
        if (this.currentState == "started") {
            // create a timer to periodically poll new results
            startTimer();
            this.currentState = "timer";
        }
    }

    /**
     * STOP
     */
    public function stopButtonClick():void {
        stopLiveMonitoring();
        this.currentState = "stopped";
    }

    /**
     * HELP BUTTON
     */
    public function showHelp():void {
        var helpWindow:MonitoringHelp = PopUpManager.createPopUp(this.graphsCanvas, MonitoringHelp, true) as MonitoringHelp;
        PopUpManager.centerPopUp(helpWindow);
    }

    /**
     * TEXT INPUT: new name of the Tab
     */
    public function changeDashboardName(newname:String):void {
        this.label = newname;
    }

    /**
     * START REPLAY from Database
     */
    protected function startDatabaseReplay():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.START_DATABASE_REPLAY + model.componentID);
        ev.ident = this.did;
        ev.from = this.databaseFrom;
        ev.to = this.databaseTo;
        dispatcher.dispatchEvent(ev);
        this.currentState = "started";
    }

    /**
     * START REPLAY from File
     */
    protected function startFileReplay():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.START_FILE_REPLAY + model.componentID);
        ev.ident = this.did;
        ev.loadFile = this.replayFileName;
        dispatcher.dispatchEvent(ev);
        this.currentState = "started";
    }

    /**
     * START Live Monitoring
     */
    protected function startLiveMonitoring():void {
        // Start live Monitoring on server
        var event:MonitoringEvent = new MonitoringEvent(MonitoringEvent.START_MONITORING + model.componentID);
        event.ident = did;
        dispatcher.dispatchEvent(event);
        this.lastState = "started";
        this.currentState = "started";
    }

    protected function stopLiveMonitoring():void {
        // First of all, stop polling data, in case the Timer is on.
        if (this.timerStarted) {
            stopTimer();
        }

        if (! replayMode) {
            // Stop live monitoring on server
            var event:MonitoringEvent = new MonitoringEvent(MonitoringEvent.STOP_MONITORING + model.componentID);
            event.ident = did;
            dispatcher.dispatchEvent(event);
        }
    }

    // -------------------------------------------------------------------
    // Global Zoom
    // -------------------------------------------------------------------

    protected var zoomFrom:Date = null;
    protected var zoomTo:Date = null;

    public function fromChanged():void {
        zoomFrom = this.dateFrom.selectedDate;
        if (zoomTo != null && zoomFrom != null) {
            this.applyZoomButton.enabled = true;
        }
    }

    public function toChanged():void {
        zoomTo = this.dateTo.selectedDate;
        if (zoomTo != null && zoomFrom != null) {
            this.applyZoomButton.enabled = true;
        }
    }

    public function setZoom():void {
        updateTimeLimits(zoomFrom, zoomTo);
        this.applyZoomButton.enabled = false;
        this.resetZoomButton.enabled = true;
    }

    public function resetZoom():void {
        updateTimeLimits(null, null);
        this.dateFrom.selectedDate = null;
        this.dateTo.selectedDate = null;
        zoomFrom = null;
        zoomTo = null;
        this.applyZoomButton.enabled = false;
        this.resetZoomButton.enabled = false;
    }

    // -----------------------------------------------------------------------------
    // private Methods
    // -----------------------------------------------------------------------------

    /**
     * Update time limits for all Pods
     * @param from
     * @param to
     */
    protected function updateTimeLimits(from:Date, to:Date):void {
        for each (var ccc:CartesianChartClass in this.graphCharts) {
            ccc.updateTimeLimits(from, to);
        }
    }

    /**
     * MENU change type of graph for current podId
     * @param type new type
     */
    protected function changeGraphType(type:int):void {

        // create a new type of chart with the same list of series
        var seriesList:ArrayCollection = new ArrayCollection();
        var serid:String = null;

        var ccc:CartesianChartClass = null;
        switch (type) {
            case LINE_CHART:
                var stroke:Stroke = new Stroke();
                stroke.weight = 1;
                ccc = new LineChartView();
                for each (serid in this.dataProviderTable[podId]) {
                    var lineSerie:LineSeries = new LineSeries();
                    lineSerie.id = serid;
                    lineSerie.dataProvider = this.seriesDataProviders[serid];
                    lineSerie.displayName = this.seriesInfo[serid].description;
                    lineSerie.yField = "value";
                    lineSerie.xField = "date";
                    lineSerie.filterData = false;
                    lineSerie.setStyle("lineStroke", stroke);
                    seriesList.addItem(lineSerie);
                }
                break;
            case AREA_CHART:
                ccc = new AreaChartView();
                for each (serid in this.dataProviderTable[podId]) {
                    var areaSerie:AreaSeries = new AreaSeries();
                    areaSerie.id = serid;
                    areaSerie.dataProvider = this.seriesDataProviders[serid];
                    areaSerie.displayName = this.seriesInfo[serid].description;
                    areaSerie.yField = "value";
                    areaSerie.xField = "date";
                    seriesList.addItem(areaSerie);
                }
                break;
        }
        ccc.setCharts(seriesList);
        this.graphCharts[podId] = ccc;

        // replace the graph inside the Pod and refresh LayoutManager
        var pod:Pod = this.graphPods[podId];
        pod.removeChildAt(INDEX_GRAPH_IN_POD);
        pod.addChildAt(ccc, INDEX_GRAPH_IN_POD);
        pod.addEventListener(FlexEvent.UPDATE_COMPLETE, displayPods);
    }

    /**
     * Parse the XML Configuration Data (strConfigFile)
     * Build all VO holding the series and the graphs to draw.
     */
    protected function parseXmlFile():Boolean {
        // Parse the XML file
        var doc:XML = new XML(this.strConfigFile);

        this.graphsCanvas.removeAllChildren();

        // flush Lists
        this.seriesInfo = new Dictionary();
        this.seriesDataProviders = new Dictionary();
        this.graphCharts = new Dictionary();
        this.graphPods = new Dictionary();
        this.dataProviderTable = new Dictionary();

        this.initPodLayoutManager();

        // for each serie
        for each (var node:XML in doc.child("serie-def")) {
            // Build a Serie VO and keep it in seriesInfo
            var serie:Serie = new Serie();
            serie.id = node.@id.toString();
            serie.name = node.name;
            serie.description = node.description;
            if (serie.description == "")
                serie.description = serie.name;
            serie.ycol = node.y.@col;
            serie.ytype = node.y.@type;
            serie.pattern = node.pattern;
            serie.server = node.server;
            this.seriesInfo[serie.id] = serie;

            // init data provider of the serie
            this.seriesDataProviders[serie.id] = new ArrayCollection();
        }

        // for each graph
        var pod:Pod = null;
        for each (var nodeG:XML in doc.child("graph")) {
            var graphId:String = nodeG.@id.toString();

            // series for the graph
            var seriesList:ArrayCollection = new ArrayCollection();


            // title
            var titleValue:String = nodeG.title;

            //type of chart
            var chartType:String = nodeG.chartType;
            var chartSelected:int;
            if (chartType == "lineChartConst") {
                chartSelected = LINE_CHART;
            }
            if (chartType == "areaChartConst") {
                chartSelected = AREA_CHART;
            }

            //create an array to add all the series for the current graph
            var graphSeries:Array = new Array;

            // add each serie for the current graph
            for each (var serieNode:XML in nodeG.child("serie")) {
                var serieId:String = serieNode.@ref.toString();

                // Check if Serie has been defined
                if (this.seriesInfo[serieId] == null) {
                    Alert.show(locale("error_missing_serie_def") + serieId);
                    return false;
                }
                graphSeries.push(serieId);

                var stroke:Stroke = new Stroke();
                stroke.weight = 1;

                switch (chartSelected) {
                    case LINE_CHART:
                        var lineSerie:LineSeries = new LineSeries();
                        lineSerie.id = serieId;
                        lineSerie.dataProvider = this.seriesDataProviders[serieId];
                        if (serieNode.toString() != "") {
                            lineSerie.displayName = serieNode.toString();
                        }
                        else {
                            lineSerie.displayName = this.seriesInfo[serieId].description;
                        }
                        lineSerie.yField = "value";
                        lineSerie.xField = "date";
                        lineSerie.setStyle("lineStroke", stroke);
                        lineSerie.filterData = false;
                        seriesList.addItem(lineSerie);
                        break;

                    case AREA_CHART:
                        var areaSerie:AreaSeries = new AreaSeries();
                        areaSerie.id = serieId;
                        areaSerie.dataProvider = this.seriesDataProviders[serieId];
                        if (serieNode.toString() != "") {
                            areaSerie.displayName = serieNode.toString();
                        }
                        else {
                            areaSerie.displayName = this.seriesInfo[serieId].description;
                        }
                        areaSerie.yField = "value";
                        areaSerie.xField = "date";
                        seriesList.addItem(areaSerie);
                        break;
                }

            }
            this.dataProviderTable[graphId] = graphSeries;

            // create the Chart
            var ccc:CartesianChartClass = null;
            switch (chartSelected) {
                case LINE_CHART:
                    ccc = new LineChartView();
                    break;
                case AREA_CHART:
                    ccc = new AreaChartView();
                    break;
            }
            ccc.setCharts(seriesList);
            this.graphCharts[graphId] = ccc;
            this.zoomOn[graphId] = false;

            // Create the pod
            pod = new Pod();
            pod.title = titleValue;
            pod.id = graphId;
            pod.addChildAt(ccc, INDEX_GRAPH_IN_POD);
            pod.addEventListener(PodMenuEvent.MENU, togglePodMenu);

            // Add Pod to the podLayoutManager
            this.podLayoutManager.addItem(pod);

            //filling the table for the correspondence between the pods and
            //the graphs
            this.graphPods[pod.id] = pod;
        }
        if (pod == null) {
            return false;
        }
        // display all the pods just created
        // We must wait that the last created Pod is completed.
        pod.addEventListener(FlexEvent.UPDATE_COMPLETE, displayPods);
        return true;
    }

    /**
     * process the new data received
     * @param provider
     */
    protected function liveDataLoad(provider:ArrayCollection):void {
        for each (var evt:EventDataVO in provider) {
            var item:Object = {date: time2date(evt.date), value: evt.value};
            (this.seriesDataProviders[evt.serid] as ArrayCollection).addItem(item);
        }
    }

    /**
     * Convert a timestamp from String to Date
     * @param time Date as a String
     * @return date as a Date
     */
    protected function time2date(time:String):Date {
        var time1:Array = time.split(" ");

        var time2:Array = time1[0].split("/");
        var year:Number = new Number(time2[0]);
        var month:Number = new Number(time2[1]) - 1;
        var day:Number = new Number(time2[2]);

        var time3:Array = time1[1].split(":");
        var hour:Number = new Number(time3[0]);
        var min:Number = new Number(time3[1]);
        var sec:Number = new Number(time3[2]);

        var date:Date = new Date(year, month, day, hour, min, sec);
        return date;
    }

    /**
     * Flush providers for this dashboard.
     */
    protected function flushProviders():void {
        for each (var provider:ArrayCollection in this.seriesDataProviders) {
            // provider.removeAll() doesn't work very well and is too costly
            provider.source = new Array();
        }
    }

    /**
     * init the podLayoutManager
     */
    protected function initPodLayoutManager():void {
        this.podLayoutManager = new PodLayoutManager();
        this.podLayoutManager.container = this.graphsCanvas;
        this.podLayoutManager.id = "unique";
    }

    /**
     * Remove this dashboard
     */
    protected function removeMe():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.REMOVE_DASHBOARD + model.componentID);
        ev.ident = this.did;
        dispatcher.dispatchEvent(ev);
        // Remove Tab
        model.util.removeTab(this as DashboardView);
    }

    /**
     * SLICER: Set the timer refresh period
     */
    protected function timerRefreshPeriodChange():void {
        setTimerPeriod(this.refreshTimeSlider.value);
    }

    protected function nextPage() {
        // Get next page of results
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.GET_LIVE_RESULT + model.componentID);
        ev.ident = did;
        dispatcher.dispatchEvent(ev);
        this.lastState = this.currentState;
        this.currentState = "waiting";
    }

    // -----------------------------------------------------------------------------
    // Monitoring timer
    // -----------------------------------------------------------------------------

    protected var timerStarted:Boolean = false;

    protected var monitoringTimer:Timer = null;

    // Current period for timer, in seconds.
    protected var timerLength:Number = 8;

    /**
     *  create a timer to periodically poll new results
     */
    protected function startTimer():void {
        if (this.monitoringTimer == null) {
            this.monitoringTimer = new Timer(this.timerLength * 1000, Infinity);
            this.monitoringTimer.addEventListener(TimerEvent.TIMER, poll);
        }
        this.monitoringTimer.start();
        // show current timer period on slicer
        this.refreshTimeSlider.value = this.timerLength;
        this.timerStarted = true;
    }

    /**
     * Callback function for Timer: must have this signature.
     * @param event
     */
    protected function poll(event:TimerEvent) {
        nextPage();
    }

    /**
     * remove the Timer
     */
    protected function stopTimer():void {
        if (this.monitoringTimer != null) {
            this.monitoringTimer.stop();
            this.monitoringTimer = null;
        }
        this.timerStarted = false;
    }

    /**
     * Set the Timer period
     */
    protected function setTimerPeriod(value:int):void {
        this.timerLength = value;

        // If timer already running, change its period accordingly
        if (this.monitoringTimer != null) {
            this.monitoringTimer.delay = this.timerLength * 1000;
        }
    }

    // -----------------------------------------------------------------------------
    // Progress Bar
    // -----------------------------------------------------------------------------

    /**
     * Title window containing the progress Bar.
     */
    protected var titleWindow:TitleWindow = null;
    protected var progressBar:ProgressBar = null;

    /**
     * Show a progress bar.
     */
    protected function showProgressBar(text:String):void {
        hideProgressBar();

        // create the progress bar.
        progressBar = new ProgressBar();
        progressBar.indeterminate = true;
        progressBar.percentWidth = 100;
        progressBar.label = text;

        // create the popup.
        titleWindow = PopUpManager.createPopUp(this.graphsCanvas, TitleWindow, true) as TitleWindow;
        titleWindow.title = locale("wait");
        titleWindow.width = this.width / 4;
        // add the progress bar to the popup
        titleWindow.addChild(progressBar);
        // center the popup.
        PopUpManager.centerPopUp(titleWindow);
    }

    /**
     * Hide the progress Bar.
     */
    protected function hideProgressBar():void {
        if (titleWindow != null) {
            // remove the popup from the screen.
            PopUpManager.removePopUp(titleWindow);
            titleWindow = null;
            progressBar = null;
        }
    }

    // -----------------------------------------------------------------------------
    // Internationalization
    // -----------------------------------------------------------------------------

    /**
     * Get localized String
     */
    public function locale(s:String):String {
        return ResourceManager.getInstance().getString("monitoringResources", s);
    }

}
}
